@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.1.1   2012/05/05
+ * Version: 0.1.9   2013/02/14
  */
 
 #ifndef _SECURITY_CAITSITH_INTERNAL_H
@@ -165,7 +165,7 @@ static inline void __list_del_entry(struct list_head *entry)
  * @member: The name of the list_struct within the struct.
  * @ss:     Pointer to "struct srcu_struct".
  *
- * As of 2.6.36, this macro is not provided because only TOMOYO wants it.
+ * As of 3.8, this macro is not provided because only CaitSith wants it.
  */
 #define list_for_each_entry_srcu(pos, head, member, ss)		      \
 	for (pos = list_entry(srcu_dereference((head)->next, ss),     \
@@ -176,6 +176,30 @@ static inline void __list_del_entry(struct list_head *entry)
 
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 5, 0)
+
+/**
+ * from_kuid - Convert kuid_t to uid_t.
+ *
+ * @ns:  Unused.
+ * @uid: kuid_t value.
+ *
+ * Returns uid seen from init's user namespace.
+ */
+#define from_kuid(ns, uid) (uid)
+
+/**
+ * from_kgid - Convert kgid_t to gid_t.
+ *
+ * @ns:  Unused.
+ * @gid: kgid_t value.
+ *
+ * Returns gid seen from init's user namespace.
+ */
+#define from_kgid(ns, gid) (gid)
+
+#endif
+
 /*
  * TOMOYO specific part start.
  */
@@ -187,6 +211,7 @@ static inline void __list_del_entry(struct list_head *entry)
 /* Index numbers for "struct ccs_condition". */
 enum ccs_conditions_index {
 	/* 0 */
+	CCS_INVALID_CONDITION,
 	CCS_SELF_UID,             /* current_uid()   */
 	CCS_SELF_EUID,            /* current_euid()  */
 	CCS_SELF_SUID,            /* current_suid()  */
@@ -196,8 +221,8 @@ enum ccs_conditions_index {
 	CCS_SELF_SGID,            /* current_sgid()  */
 	CCS_SELF_FSGID,           /* current_fsgid() */
 	CCS_SELF_PID,             /* sys_getpid()   */
-	CCS_SELF_PPID,            /* sys_getppid()  */
 	/* 10 */
+	CCS_SELF_PPID,            /* sys_getppid()  */
 	CCS_TASK_TYPE,            /* ((u8) task->ccs_flags) &
 				     CCS_TASK_IS_EXECUTE_HANDLER */
 	CCS_SELF_DOMAIN,
@@ -208,8 +233,8 @@ enum ccs_conditions_index {
 	CCS_OBJ_IS_SYMLINK,       /* S_IFLNK */
 	CCS_OBJ_IS_FILE,          /* S_IFREG */
 	CCS_OBJ_IS_BLOCK_DEV,     /* S_IFBLK */
-	CCS_OBJ_IS_DIRECTORY,     /* S_IFDIR */
 	/* 20 */
+	CCS_OBJ_IS_DIRECTORY,     /* S_IFDIR */
 	CCS_OBJ_IS_CHAR_DEV,      /* S_IFCHR */
 	CCS_OBJ_IS_FIFO,          /* S_IFIFO */
 	CCS_MODE_SETUID,          /* S_ISUID */
@@ -219,8 +244,8 @@ enum ccs_conditions_index {
 	CCS_MODE_OWNER_WRITE,     /* S_IWUSR */
 	CCS_MODE_OWNER_EXECUTE,   /* S_IXUSR */
 	CCS_MODE_GROUP_READ,      /* S_IRGRP */
-	CCS_MODE_GROUP_WRITE,     /* S_IWGRP */
 	/* 30 */
+	CCS_MODE_GROUP_WRITE,     /* S_IWGRP */
 	CCS_MODE_GROUP_EXECUTE,   /* S_IXGRP */
 	CCS_MODE_OTHERS_READ,     /* S_IROTH */
 	CCS_MODE_OTHERS_WRITE,    /* S_IWOTH */
@@ -228,7 +253,6 @@ enum ccs_conditions_index {
 	CCS_TASK_EXECUTE_HANDLER, /* CCS_TASK_IS_EXECUTE_HANDLER */
 	CCS_HANDLER_PATH,
 	CCS_TRANSIT_DOMAIN,
-	CCS_MAX_CONDITION_KEYWORD,
 	CCS_COND_SARG0,
 	CCS_COND_SARG1,
 	/* 40 */
@@ -365,7 +389,62 @@ enum ccs_mac_index {
 #ifdef CONFIG_CAITSITH_MANUAL_DOMAIN_TRANSITION
 	CCS_MAC_MANUAL_DOMAIN_TRANSITION,
 #endif
-	CCS_MAX_MAC_INDEX
+	CCS_MAX_MAC_INDEX,
+	/* Map undefined functions to CCS_MAX_MAC_INDEX */
+#ifndef CONFIG_CAITSITH_GETATTR
+	CCS_MAC_GETATTR = CCS_MAX_MAC_INDEX,
+#endif
+#ifndef CONFIG_CAITSITH_NETWORK
+	CCS_MAC_INET_STREAM_BIND = CCS_MAX_MAC_INDEX,
+	CCS_MAC_INET_STREAM_LISTEN = CCS_MAX_MAC_INDEX,
+	CCS_MAC_INET_STREAM_CONNECT = CCS_MAX_MAC_INDEX,
+	CCS_MAC_INET_STREAM_ACCEPT = CCS_MAX_MAC_INDEX,
+	CCS_MAC_INET_DGRAM_BIND = CCS_MAX_MAC_INDEX,
+	CCS_MAC_INET_DGRAM_SEND = CCS_MAX_MAC_INDEX,
+	CCS_MAC_INET_RAW_BIND = CCS_MAX_MAC_INDEX,
+	CCS_MAC_INET_RAW_SEND = CCS_MAX_MAC_INDEX,
+	CCS_MAC_UNIX_STREAM_BIND = CCS_MAX_MAC_INDEX,
+	CCS_MAC_UNIX_STREAM_LISTEN = CCS_MAX_MAC_INDEX,
+	CCS_MAC_UNIX_STREAM_CONNECT = CCS_MAX_MAC_INDEX,
+	CCS_MAC_UNIX_STREAM_ACCEPT = CCS_MAX_MAC_INDEX,
+	CCS_MAC_UNIX_DGRAM_BIND = CCS_MAX_MAC_INDEX,
+	CCS_MAC_UNIX_DGRAM_SEND = CCS_MAX_MAC_INDEX,
+	CCS_MAC_UNIX_SEQPACKET_BIND = CCS_MAX_MAC_INDEX,
+	CCS_MAC_UNIX_SEQPACKET_LISTEN = CCS_MAX_MAC_INDEX,
+	CCS_MAC_UNIX_SEQPACKET_CONNECT = CCS_MAX_MAC_INDEX,
+	CCS_MAC_UNIX_SEQPACKET_ACCEPT = CCS_MAX_MAC_INDEX,
+#endif
+#ifndef CONFIG_CAITSITH_NETWORK_RECVMSG
+	CCS_MAC_INET_DGRAM_RECV = CCS_MAX_MAC_INDEX,
+	CCS_MAC_INET_RAW_RECV = CCS_MAX_MAC_INDEX,
+	CCS_MAC_UNIX_DGRAM_RECV = CCS_MAX_MAC_INDEX,
+#endif
+#ifndef CONFIG_CAITSITH_ENVIRON
+	CCS_MAC_ENVIRON = CCS_MAX_MAC_INDEX,
+#endif
+#ifndef CONFIG_CAITSITH_PTRACE
+	CCS_MAC_PTRACE = CCS_MAX_MAC_INDEX,
+#endif
+#ifndef CONFIG_CAITSITH_SIGNAL
+	CCS_MAC_SIGNAL = CCS_MAX_MAC_INDEX,
+#endif
+#ifndef CONFIG_CAITSITH_CAPABILITY
+	CCS_MAC_USE_NETLINK_SOCKET = CCS_MAX_MAC_INDEX,
+	CCS_MAC_USE_PACKET_SOCKET = CCS_MAX_MAC_INDEX,
+	CCS_MAC_USE_REBOOT = CCS_MAX_MAC_INDEX,
+	CCS_MAC_USE_VHANGUP = CCS_MAX_MAC_INDEX,
+	CCS_MAC_SET_TIME = CCS_MAX_MAC_INDEX,
+	CCS_MAC_SET_PRIORITY = CCS_MAX_MAC_INDEX,
+	CCS_MAC_SET_HOSTNAME = CCS_MAX_MAC_INDEX,
+	CCS_MAC_USE_KERNEL_MODULE = CCS_MAX_MAC_INDEX,
+	CCS_MAC_USE_NEW_KERNEL = CCS_MAX_MAC_INDEX,
+#endif
+#ifndef CONFIG_CAITSITH_AUTO_DOMAIN_TRANSITION
+	CCS_MAC_AUTO_DOMAIN_TRANSITION = CCS_MAX_MAC_INDEX,
+#endif
+#ifndef CONFIG_CAITSITH_MANUAL_DOMAIN_TRANSITION
+	CCS_MAC_MANUAL_DOMAIN_TRANSITION = CCS_MAX_MAC_INDEX,
+#endif
 } __packed;
 
 /* Index numbers for statistic information. */
@@ -415,7 +494,7 @@ enum ccs_policy_stat_type {
 	CCS_MAX_POLICY_STAT
 } __packed;
 
-/* Index numbers for /proc/ccs/ interfaces. */
+/* Index numbers for /proc/caitsith/ interfaces. */
 enum ccs_proc_interface_index {
 	CCS_POLICY,
 	CCS_PROCESS_STATUS,
@@ -447,15 +526,6 @@ enum ccs_value_type {
 	CCS_VALUE_TYPE_HEXADECIMAL,
 } __packed;
 
-/* Index numbers for type of IP addresses. */
-enum ccs_ipaddr_type {
-	CCS_ADDRESS_TYPE_INVALID,
-	CCS_ADDRESS_TYPE_IPV4,
-	CCS_ADDRESS_TYPE_IPV4_RANGE,
-	CCS_ADDRESS_TYPE_IPV6,
-	CCS_ADDRESS_TYPE_IPV6_RANGE,
-} __packed;
-
 /* Constants definition for internal use. */
 
 /*
@@ -487,7 +557,7 @@ enum ccs_ipaddr_type {
 #define CCS_TASK_IS_IN_EXECVE             4
 /* Current thread is running as an execute handler program? */
 #define CCS_TASK_IS_EXECUTE_HANDLER       8
-/* Current thread is allowed to modify policy via /proc/ccs/ interface? */
+/* Current thread is allowed to modify policy via /proc/caitsith/ interface? */
 #define CCS_TASK_IS_MANAGER              16
 
 /*
@@ -499,7 +569,7 @@ enum ccs_ipaddr_type {
  */
 #define CCS_RETRY_REQUEST 1
 
-/* Size of read buffer for /proc/ccs/ interface. */
+/* Size of read buffer for /proc/caitsith/ interface. */
 #define CCS_MAX_IO_READ_QUEUE 64
 
 /* Structure definition for internal use. */
@@ -562,8 +632,13 @@ struct ccs_ip_group {
 
 /* Subset of "struct stat". Used by conditional ACL and audit logs. */
 struct ccs_mini_stat {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
+	kuid_t uid;
+	kgid_t gid;
+#else
 	uid_t uid;
 	gid_t gid;
+#endif
 	ino_t ino;
 	umode_t mode;
 	dev_t dev;
@@ -655,7 +730,7 @@ struct ccs_request_info {
 	 */
 	struct ccs_path_info exename;
 	/*
-	 * Matching "struct ccs_acl_info" is copied. Used for ccs-queryd.
+	 * Matching "struct ccs_acl_info" is copied. Used for caitsith-queryd.
 	 * Valid until ccs_read_unlock().
 	 */
 	struct ccs_acl_info *matched_acl;
@@ -705,7 +780,7 @@ struct ccs_name {
 	struct ccs_path_info entry;
 };
 
-/* Structure for reading/writing policy via /proc/ccs/ interfaces. */
+/* Structure for reading/writing policy via /proc/caitsith/ interfaces. */
 struct ccs_io_buffer {
 	/* Exclusive lock for this structure.   */
 	struct mutex io_sem;
@@ -1082,27 +1157,21 @@ static inline u32 ccs_current_flags(void)
  */
 #define ccs_security task_struct
 
-/**
- * ccs_find_task_security - Find "struct ccs_security" for given task.
- *
- * @task: Pointer to "struct task_struct".
- *
- * Returns pointer to "struct ccs_security".
- */
-static inline struct ccs_security *ccs_find_task_security(struct task_struct *
-							  task)
-{
-	return task;
-}
-
 /**
  * ccs_current_security - Get "struct ccs_security" for current thread.
  *
  * Returns pointer to "struct ccs_security" for current thread.
+ *
+ * If current thread does not belong to a domain (which is true for initial
+ * init_task in order to hide ccs_kernel_domain from this module),
+ * current thread enters into ccs_kernel_domain.
  */
 static inline struct ccs_security *ccs_current_security(void)
 {
-	return ccs_find_task_security(current);
+	struct task_struct *task = current;
+	if (!task->ccs_domain_info)
+		task->ccs_domain_info = &ccs_kernel_domain;
+	return task;
 }
 
 /**
@@ -1122,17 +1191,10 @@ static inline struct ccs_domain_info *ccs_task_domain(struct task_struct *task)
  * ccs_current_domain - Get "struct ccs_domain_info" for current thread.
  *
  * Returns pointer to "struct ccs_domain_info" for current thread.
- *
- * If current thread does not belong to a domain (which is true for initial
- * init_task in order to hide ccs_kernel_domain from this module),
- * current thread enters into ccs_kernel_domain.
  */
 static inline struct ccs_domain_info *ccs_current_domain(void)
 {
-	struct task_struct *task = current;
-	if (!task->ccs_domain_info)
-		task->ccs_domain_info = &ccs_kernel_domain;
-	return task->ccs_domain_info;
+	return ccs_current_security()->ccs_domain_info;
 }
 
 /**
@@ -1144,7 +1206,7 @@ static inline struct ccs_domain_info *ccs_current_domain(void)
  */
 static inline u32 ccs_task_flags(struct task_struct *task)
 {
-	return ccs_find_task_security(task)->ccs_flags;
+	return task->ccs_flags;
 }
 
 /**
@@ -1154,7 +1216,7 @@ static inline u32 ccs_task_flags(struct task_struct *task)
  */
 static inline u32 ccs_current_flags(void)
 {
-	return ccs_find_task_security(current)->ccs_flags;
+	return current->ccs_flags;
 }
 
 #endif
