@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 0.1.3   2012/08/02
+ * Version: 0.1.9   2013/02/14
  */
 
 #include <linux/version.h>
@@ -155,6 +155,37 @@ static void ccs_load_policy(const char *filename)
 
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 8, 0)
+
+/**
+ * __ccs_search_binary_handler - Load policy before calling search_binary_handler().
+ *
+ * @bprm: Pointer to "struct linux_binprm".
+ *
+ * Returns 0 on success, negative value otherwise.
+ */
+static int __ccs_search_binary_handler(struct linux_binprm *bprm)
+{
+#ifndef CONFIG_CAITSITH_OMIT_USERSPACE_LOADER
+	ccs_load_policy(bprm->filename);
+#endif
+	/*
+	 * ccs_load_policy() executes /sbin/caitsith-init if bprm->filename is
+	 * /sbin/init. /sbin/caitsith-init executes
+	 * /etc/caitsith/caitsith-load-module to load loadable kernel module.
+	 * The loadable kernel module modifies "struct caitsith_ops". Thus,
+	 * we need to transfer control to __ccs_search_binary_handler() in
+	 * security/caitsith/permission.c if "struct caitsith_ops" was
+	 * modified.
+	 */
+	if (caitsith_ops.search_binary_handler
+	    != __ccs_search_binary_handler)
+		return caitsith_ops.search_binary_handler(bprm);
+	return search_binary_handler(bprm);
+}
+
+#else
+
 /**
  * __ccs_search_binary_handler - Load policy before calling search_binary_handler().
  *
@@ -172,7 +203,7 @@ static int __ccs_search_binary_handler(struct linux_binprm *bprm,
 	/*
 	 * ccs_load_policy() executes /sbin/caitsith-init if bprm->filename is
 	 * /sbin/init. /sbin/caitsith-init executes
-	 * /etc/ccs/caitsith-load-module to load loadable kernel module.
+	 * /etc/caitsith/caitsith-load-module to load loadable kernel module.
 	 * The loadable kernel module modifies "struct caitsith_ops". Thus,
 	 * we need to transfer control to __ccs_search_binary_handler() in
 	 * security/caitsith/permission.c if "struct caitsith_ops" was
@@ -184,6 +215,8 @@ static int __ccs_search_binary_handler(struct linux_binprm *bprm,
 	return search_binary_handler(bprm, regs);
 }
 
+#endif
+
 /*
  * Some exports for loadable kernel module part.
  *
